/**********************************************************************
  Filename    : Camera Remote Bluetooth IoT
  Description : Control android app Camera Remote Bluetooth remotely.
                Change camera modes: photo or video.
                Change flash modes: torch or auto
                Record video.
                Take photo.
                Switch camera: front, rear camera
                Change display modes: none, text, image
                Change color modes: 65k (VGA), 256 (CGA) mode
  Version     : ESP32 WROOM and for TFT ST7789 240x240 Color Display            
  Auther      : www.busywww.com   busyweb@gmail.com
  Modification: 2021/10/30

// Jpeg Decorder: Adafruit_GFX compatible example for library:
// https://github.com/Bodmer/TJpg_Decoder 
**********************************************************************/
#include "soc/soc.h"
#include "soc/rtc_cntl_reg.h"

#include "BluetoothSerial.h"
#include "string.h"

#include <Arduino.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
//#include <Adafruit_SSD1306.h>
#include <Adafruit_ST7789.h> // Hardware-specific library for ST7789

#include "imagelogo.h"
#include "imageicon.h"
#include "variables.h"
#include "functions.h"

// This next function will be called during decoding of the jpeg file to
// render each block to the TFT.  If you use a different TFT library
// you will need to adapt this function to suit.
bool tft_output(int16_t x, int16_t y, uint16_t w, uint16_t h, uint16_t* bitmap)
{
   // Stop further decoding as image is running off bottom of screen
  if ( y >= tft.height() ) return 0;

  // In ILI9341 library this function clips the image block at TFT boundaries
  tft.drawRGBBitmap(x, y, bitmap, w, h);

  // Return 1 to decode next block
  return 1;
}

void setup() {

  tft.init(240, 240, SPI_MODE3);           // Init ST7789 240x240, SPI_MODE3
  tft.setSPISpeed(40000000);
  
  pinMode(PIN_LED_PHOTO, OUTPUT);
  pinMode(PIN_LED_VIDEO, OUTPUT);

  // Switches Mode
  pinMode(PIN_BUTTON_PHOTO, INPUT_PULLUP);
  pinMode(PIN_BUTTON_VIDEO, INPUT_PULLUP);
  pinMode(PIN_BUTTON_FLASH, INPUT_PULLUP);

  // Action Button
  pinMode(BUTTON_ACTION, INPUT_PULLUP);

  // Change Camera
  pinMode(BUTTON_CHANGE_CAMERA, INPUT_PULLUP);
  
  // Change Display Mode: Text, Information, and Image
  pinMode(BUTTON_DISPLAY_MODE, INPUT_PULLUP);


  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 0);    //disable brownout detector   ds

  // Bluetooth device name which will be listed on the remote android bluetooth paired (bonded) list
  // For Menu version, set master mode true is important...
  // if set to true on Master mode bluetooth, remote camera would not connect to remote controller...
  SerialBT.begin("Camera Remote Bluetooth", useMenu);    //Bluetooth device name 
//  SerialBT.begin("Camera Remote Bluetooth");
  Serial.begin(115200);

  SerialBT.register_callback(callback);
  
  Serial.println("\nThe device started, now you can pair it with bluetooth!");
  
  delay(2000);
  WRITE_PERI_REG(RTC_CNTL_BROWN_OUT_REG, 1);    //enable brownout detector

  // TFT: ST7789
  displayOk = true;
  Serial.println(F("Display Mode: ST7789"));
  // landscape mode (1, 3), portrait mode (0, 2)
  tft.setRotation(2);
  tft.fillScreen(ST77XX_BLACK);
  
  // Draw logo image
  displayLogo();

  MODE_DISPLAY = 0;
  imageDataProcessing = false;

  // allocate memory for the remote image frame data
  prepareImageDataReceiving();

  delay(500);
  
  TJpgDec.setJpgScale(1);
  // The decoder must be given the exact name of the rendering function above
  TJpgDec.setCallback(tft_output);
}

void loop() {

//  ST7789 Implementation ***************************************
  currentTime = millis();
  if (modeChangedTime < 0) {
    modeChangedTime = currentTime;
  }

  if (CONNECTED) {

    while(SerialBT.available())
    {

      if (!imageDataProcessing) {
      
          String received = SerialBT.readStringUntil('\n');
         
          if (received.startsWith("imagedata")) {
            imageDataProcessing = true;            
            processImageData(received);
          } else {
            //imageDataProcessing = false;
            if (MODE_DISPLAY != 2) {
              processData(received);   
            }
          }
            
      }
      
    }

    processAction();

  } else { //end of: if (CONNECTED)

    processAction();


    // not connected
    digitalWrite(PIN_LED_PHOTO, LOW);
    digitalWrite(PIN_LED_VIDEO, LOW);
    
  }

  delay(10);
}

// Jpeg decoding example code
//void loop2() {
//  // Time recorded for test purposes
//  uint32_t t = millis();
//
//  // Get the width and height in pixels of the jpeg if you wish
//  uint16_t w = 0, h = 0;
//  TJpgDec.getJpgSize(&w, &h, sample, sizeof(sample));
//  Serial.print("Width = "); Serial.print(w); Serial.print(", height = "); Serial.println(h);
//
//    // Draw the image, top left at 0,0
//  TJpgDec.drawJpg(0, 0, sample, sizeof(sample));
//
//    // How much time did rendering take (ESP8266 80MHz 473ms, 160MHz 266ms, ESP32 SPI 116ms)
//  t = millis() - t;
//  Serial.print(t); Serial.println(" ms");
//
//  // Wait before drawing again
//  delay(2000);
//}
